
From: olof@austin.ibm.com (Olof Johansson)

For prefetches of NULL (as when walking a short linked list), PPC64 will in
some cases take a performance hit.  The hardware needs to do the TLB walk,
and said walk will always miss, which means (up to) two L2 misses as
penalty.  This seems to hurt overall performance, so for NULL pointers skip
the prefetch alltogether.

Signed-off-by: Olof Johansson <olof@austin.ibm.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/asm-ppc64/processor.h |    6 ++++++
 1 files changed, 6 insertions(+)

diff -puN include/asm-ppc64/processor.h~ppc64-no-prefetch-for-null-pointers include/asm-ppc64/processor.h
--- 25/include/asm-ppc64/processor.h~ppc64-no-prefetch-for-null-pointers	2005-04-10 16:30:22.000000000 -0700
+++ 25-akpm/include/asm-ppc64/processor.h	2005-04-10 16:30:22.000000000 -0700
@@ -640,11 +640,17 @@ static inline unsigned long __pack_fe01(
 
 static inline void prefetch(const void *x)
 {
+	if (unlikely(!x))
+		return;
+
 	__asm__ __volatile__ ("dcbt 0,%0" : : "r" (x));
 }
 
 static inline void prefetchw(const void *x)
 {
+	if (unlikely(!x))
+		return;
+
 	__asm__ __volatile__ ("dcbtst 0,%0" : : "r" (x));
 }
 
_
